/*
     RAEAN-1b: NormsModel.java
  Copyright (C) 2004  Universidad de Burgos

 This file is part of RAEAN (Re-implementation of Axelrod's "Evolutionary approach to norms"),
 an agent-based model designed to replicate and extend Prof. Axelrods
 agent-based model described in the paper "An evolutionary approach to
 norms", published in American Political Science Review 80, n4,
 December 1996: 1095-1111, and reprinted in chapter 3 (Promoting
 norms) of Robert M. Axelrods book "The Complexity of Cooperation.
 Agent-Based Models of Competition and Collaboration", Princeton
 University Press. Princeton, New Jersey 1997.

 The code is written in Java 2, and is known to work using RePast-2.2
 libraries (http://repast.sourceforge.net/) and JDK 1.4.2_01-b06
 distribution on a PC running Windows 2000, NT and XP.

     RAEAN is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     RAEAN is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details. (LICENCE file in
     this directory.)

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

     Contact information:
       Jos M. Galn,
       Universidad de Burgos, Burgos, Spain
       jmgalan@ubu.es
 */


import cern.jet.random.Uniform;
import java.util.Date;
import java.io.*;
import uchicago.src.sim.engine.Schedule;
import uchicago.src.sim.engine.SimModelImpl;
import uchicago.src.sim.analysis.*;
import uchicago.src.sim.engine.*;
import uchicago.src.reflector.ListPropertyDescriptor;
import java.util.ArrayList;
import uchicago.src.reflector.ListPropertyDescriptor;
import uchicago.src.sim.util.ProbeUtilities;
import java.util.Vector;
import uchicago.src.sim.engine.Controller;

/**
 * <p>Title: Re-implementation of Axelrod's "Evolutionary approach to norms"</p>
 * <p>Description: Reimplementation by Jos M. Galn & Luis R. Izquierdo</p>
 * <p>Copyright: Universidad de Burgos</p>
 * <p>Company: Universidad de Burgos & Macauley Institute & INSISOC Group</p>
 * @author Jos Manuel Galn & Luis R. Izquierdo (in alphabetic order)
 * @version 1.0
 */
public class NormsModel
    extends SimModelImpl {

  /**
   * This parameter represents the number of players in the game. It must be a positive integer
   */
  private int numAgents;

  /**
   * The characteristics of the agents in Axelrods original model were represented by a 3-bit string to allow eight different levels.
   * In our program the parameter ArrayDimension determines the number of bits representing each of the characteristics of the agents.
   * A higher value of the parameter involves more different levels of boldness, vengefulness or metavengefulness available for the players.
   */
  private int arrayDimension;

  /**
   *This parameter is a double that determines the payoff initially obtained by a defecting agent (T  = Temptation to defect).
   */
  private double defection_T;

  /**
   * If agent i defects, some of the other agents may see the defection, and those who do may choose to punish (P) the defector.
   * A defector receives a Punishment_P payoff every time it is punished. Like all the payoff parameters in the program it is represented with a double, and since it represents a punishment it usually has a negative value
   */
  private double punishment_P;

  /**
   *If an agent i defects, it obtains Defection_T payoff, but each of the others are hurt (H), getting a payoff with the value of the parameter HurtByOthers_H.
   * Please note that this parameter has typically a negative value.
   * It is also implemented as a double.
   */
  private double hurtByOthers_H;

  /**
   * If an agent j decides to punish an agent i who has defected, agent i obtains the Punishment_P payoff but because the act of punishing is typically somewhat costly, the punisher (agent j), has to pay an enforcement cost (E).
   * The enforcement cost is represented by a double in the EnforcementCost_E parameter and it is typically a negative number.
   */
  private double enforcementCost_E;

  /**
   * This parameter has influence only in the metanorms model (activateMetanormsModel must be true). In the metanorms game, if someone defects (agent i), and another agent (agent j) sees that defection but does not punish it, then the other players (excluding agent i and agent j) have a chance to see and to punish agent j.
   * If a non-punisher (agent j) is punished, it receives a PunishmentAgainstNonpunishment_PNP payoff.
   * Like all the payoff parameters in the program it is represented with a double, and since it represents a punishment it usually has a negative value.
   */
  private double punishmentAgainstNonpunishment_PNP;

  /**
   * This parameter is only relevant in the metanorms game. Following the same nomenclature that in the preceding paragraph, when an agent k has seen an agent j that has not punished a defector i, and k decides to punish j, the agent punished j receives a PunishmentAgainstNonpunishment_PNP payoff, but agent k has to pay an enforcement cost against the non punishment (ENP).
   * This cost is represented by a double in the EnforcementCostAgainstNonpunishment_ENP parameter and it is typically a negative number.
   */
  private double enforcementCostAgainstNonpunishment_ENP;

  /**
   * This parameter is a double between 0 and 1. It is the probability that each of the bits representing an agents strategy (boldness, vengefulness and maybe metaVengefulness) will be flipped every time an agent is replicated.
   */
  private double mutationRate_per_bit;

  /**
   * This parameter is the expected value of mutations (bits flipped) per generation. It is linearly related with MutationRate_per_bit. Setting one of them completely determines the other. If we fix the MutationRate_per_bit parameter, then MutationRate_per_generation is calculated with the following code:

   if (metaVengefulnessEqualToVengefulness) {
      mutationRate_per_generation = mutationRate_per_bit * (numAgents *
          (arrayDimension * 2));
    }
    else {
      mutationRate_per_generation = mutationRate_per_bit * (numAgents *
          (arrayDimension * 3));
    }

   Consequently if we set up the MutationRate_per_generation then the MutationRate_per_generation is:


   if (metaVengefulnessEqualToVengefulness) {
      mutationRate_per_bit = mutationRate_per_generation / (numAgents *
          arrayDimension * 2);
    }
    else {
      mutationRate_per_bit = mutationRate_per_generation / (numAgents *
          arrayDimension * 3);

    }

   */
  private double mutationRate_per_generation;

  /**
   * This parameter establishes the number of standard deviations plus the population average payoff that an agent needs to obtain to be replicated twice in the selection method of STANDARD_DEVIATION
   */
  private double standardDeviationsForDoubleReproduction;

  /**
   * That parameter establishes the number of standard deviations plus the population average payoff that an agent needs to obtain to be replicated at least once in the selection method of STANDARD_DEVIATION
   */
  private double standardDeviationsForNoReproduction;

  /**
   * This parameter is only used when the selection mechanism is ROULETTE_WHEEL. Such method involves calculating every agents fitness, which is equal to their payoff minus the minimum payoff obtained in the generation plus ConstantForRouletteWheel (double).
   * Fitness[i]=Score[i]+minScoreInThePopulation+ ConstantForRouletteWheel
   */
  private double constantForRouletteWheel;

  /**
   * This parameter has two different interpretations (see Initialization parameter):
   *
   * 1.	In case that initialization has the value RANDOM, InitialBoldnessAverage shows the expected average value of the initial boldness of the population, and its value can not be modified.
   * 2.	In case that initialization has the value CUSTOM, InitialBoldnessAverage allows customizing the boldness of all the agents in the initial population (All the agents with the value determined by the user, but all with the same value).
   */
  public double initialBoldnessAverage;

  /**
   * Analogous to InitialBoldnessAverage.
   */
  public double initialVengefulnessAverage;

  /**
   * This parameter has three different interpretations (see Initialization parameter):
   *
   * 1.	If initialization is RANDOM, the parameter InitialMetaVengefulnessAverage shows the expected average value of the initial metavengefulness of the population, and its value can not be modified.
   * 2.	If initialization is CUSTOM and the parameter metaVengefulnessEqualToVengefulness is true, then InitialMetaVengefulnessAverage will have the same value as InitialVengefulnessAverage.
   * 3.	If initialization is CUSTOM and the parameter MetaVengefulnessEqualToVengefulness false, then InitialMetaVengefulnessAverage and InitialVengefulnessAverage parameters are decoupled and the user can set up the metaVengefulness of all the agents in the initial population (All the agents with the value determined by the user, but all with the same value).
   *
   */
  public double initialMetaVengefulnessAverage;

  /**
   * This parameter represents the number of rounds that the game is played in each generation.
   */
  private int rounds;

  /**
   * This parameter determines the number of generations to run. It partially determines the length of the simulation. We can consider that a single simulation is divided into four steps.  The NumberOfGenerations parameter sets the number of times that a new population of players is created according to their relative scores.
   */
  private int numberOfGenerations;

  /**
   * RePast schedule
   */
  private Schedule schedule;

  /**
   * List of agents
   */
  private ArrayList normsAgentList = new ArrayList();

  /**
   * Average score of the population in a generation
   */
  private double averageScore = 0;
  private double sumaTotal = 0;
  /**
   * Standard deviation of the population in a generation
   */
  private double deviation = 0;

  /**
   * Graph with the temporal evolution of the agents average properties: average boldness, average vengefulness and, if appropriate, average metavengefulness
   */
  private OpenSequenceGraph graph;

  /**
   *2.	A  plot that shows dynamically the evolution of the properties in a boldness average (X axis) - vengefulness average (Y axis) map. This map is intended to observe easily the proportion of time that the system spends in states of norm collapse and in states where the norm is established, and how the evolutionary transition from the former to the latter occurs.
   */
  private Plot aPlot;

  /**
   * Average boldness of the population in a time period
   */
  private double averageBoldness = 0;

  /**
   * Average vengefulness of the population in a time period
   */
  private double averageVengefulness = 0;

  /**
   * Average metavengefulness of the population in a time period
   */
  private double averageMetaVengefulness = 0;

  /**
   * In the current version of the implementation there are two different methods available to initialise the characteristics of the population:
   * 1.	The first one is RANDOM. This is the method used in Axelrods original implementation. It consists in generating the initial boldness, vengefulness and metavengefulness of each agent randomly with a uniform distribution among the values set. If this initialization method is selected, the parameters initialBoldnessAverage, initialVengefulnessAverage and initialMetaVengefulnessAverage can not be modified, and their probes show their expected values
   * 2.	The second initialization method is CUSTOM. This method is deterministic. It allows setting up a population with the specific values preferred by a user. The only constraint is that the initial characteristics of the agents are the same for every agent. In that case the parameters initialBoldnessAverage, initialVengefulnessAverage and initialMetaVengefulnessAverage determine the specified values of the initial population. Please note that the simulator (in the discrete version) does not allow setting up a non integer value in this parameter. The simulator also checks if the parameter metaVengefulnessEqualToVengefulness is activated, and if so, it ensures that both parameters are equal. If the user deactivate metaVengefulnessEqualToVengefulness, then it is possible to set different values for the two parameters.
   */
  private String initialization;

  /**
   * In our program there are three selection mechanisms available:
   1.	STANDARD_DEVIATION . This method is a generalisation of the one implemented by Axelrod. Under this selection mechanism agents with a payoff equal to or greater than the population average plus StandardDeviationForDoubleReproduction standard deviations are replicated twice; agents who are below the population average plus  StandardDeviationForNoReproduction standard deviations are eliminated; and the rest of the agents are replicated once. .
   2.	RANDOM_TOURNAMENT. This method involves selecting two agents from the population at random and replicating the one with the higher payoff for the next generation. If case of tie, one of them is selected at random. This process is repeated as many times as necessary to keep the number of agents constant (NumAgents).
   3.	ROULETTE_WHEEL. This method involves calculating every agents fitness, which is equal to their payoff minus the minimum payoff obtained in the generation plus a parameter called ConstantForRouletteWheel . Agents are then given a probability of being replicated (in each of the NumAgents replications) that is directly proportionate to their fitness. If every agent obtained exactly the same score, we execute a random tournament for that generation.
   */
  private String selection;

  /**
   * Variable that contains the name of inputs of the program
   */
  public String parametersUsed;

  /**
   * Variable that contains the name of outputs of the program (the data recorder)
   */
  public String outputFileName;

  /**
   * This parameter is represented by a boolean variable. It represents each one of two types of models that can be run in the program:
   * 1.	Axelrods norms game, if the parameter is false. In the GUI mode if the check box is not activated.
   * 2.	Axelrods metanorms game if the parameter is true (if it is activated).
   */
  private boolean activateMetanormsModel;

  /**
   This parameter is represented by a boolean variable. If it is true (its check box is activated), the metanorms game includes Axelrods hypothesis that an agents propensity to punish a defection (vengefulness) is the same as its propensity to punish to a non-punisher (metavengefulness).
   If false, the variables vengefulness and metaVengefulness will not be necessarily equal.
   */
  private boolean metaVengefulnessEqualToVengefulness;

  /**
   * If ParamOutputFile is true (if its check box is activated) the program creates a .csv  file with the relevant parameters after each execution. Only the parameters used in the run are printed to file.
   * For instance, if the selected selection mechanism is STANDARD_DEVIATION, the parameter ConstantForRouletteWheel will not be included in the file because it is associated with the ROULETTE_WHEEL selection mechanism.
   */
  private boolean paramOutputFile;

  /**
   * If this parameter is true, the population averages of boldness, vengefulness and metavengefulness (if it is different from the vengefulness) are stored in a .txt file while the simulation is running. As in the previous case, the file is created in the folder where the program is, and its name includes the word output, the date, and the time of the beginning of the run.

   If the ParamInputFile and DataOutputFile parameters are both activated, the .csv and .txt files will contain exactly the same date and time.

   */
  private boolean dataOutputFile;

  /**
   * Strings that contains the file name of the parameters in the batch mode
   */
  public String parameterFileName;

  /**
   * This parameter is related with the DataOutputFile parameter. It determines the frequency of data saving to the data file. For instance, if the value of the parameter is 2, the population averages are recorded every two generations.
   If DataOutputFile is false (not activated), the OutputStep has no influence in the program.
   */
  public int outputStep;

  /**
   * Determine if the simulation is going to be run in batch (true) or GUI mode (false)
   */
  public boolean batchMode;

  /**
   * To write in the input file
   */
  FileWriter fw2;
  /**
   * To write in the input file
   */
  BufferedWriter bw2;
  /**
   * To write in the input file
   */
  PrintWriter salida2;

  /**
   * Contructor of the NormsModel class
   */
  public NormsModel() {

    Vector v = new Vector();
    v.add("RANDOM");
    v.add("CUSTOM");

    Vector v2 = new Vector();
    v2.add("STANDARD_DEVIATION");
    v2.add("ROULETTE_WHEEL");
    v2.add("RANDOM_TOURNAMENT");

    if (metaVengefulnessEqualToVengefulness) {
      this.mutationRate_per_generation = this.mutationRate_per_bit *
          (numAgents * arrayDimension * 2);

    }
    else {

      this.mutationRate_per_generation = this.mutationRate_per_bit *
          (numAgents * arrayDimension * 3);
    }

    ListPropertyDescriptor pd = new ListPropertyDescriptor("Initialization", v);
    descriptors.put("Initialization", pd);

    ListPropertyDescriptor pd2 = new ListPropertyDescriptor("Selection", v2);
    descriptors.put("Selection", pd2);

    Controller.ALPHA_ORDER = false;
  }

  /**
   * This method creates and intializes the first generation of agents depending on the initialization method selected. It also creates the data and param file if necessary.s
   */
  private void buildModel() {

    if (this.initialization.equals("RANDOM")) {
      for (int i = 0; i < numAgents; i++) {

        NormsAgent anAgent = new NormsAgent(arrayDimension, rounds);
        this.normsAgentList.add(anAgent);
      }
    }

    if (this.initialization.equals("CUSTOM")) {
      for (int i = 0; i < numAgents; i++) {

        NormsAgent anAgent = new NormsAgent(arrayDimension, rounds,
                                            this.getInitialBoldnessAverage(),
                                            this.getInitialVengefulnessAverage(),
                                            this.
                                            getInitialMetaVengefulnessAverage());
        this.normsAgentList.add(anAgent);
      }
    }

    if (this.getDataOutputFile() || this.getParamOutputFile()) {
      this.createNameFile();

      if (this.getParamOutputFile())
        this.writeParams();
    }

  }

  /**
   * This method builds the sequence plot and the temporal graph in the GUI mode
   */
  private void buildDisplay() {
//graph configuration
    graph.addSequence("Boldness Average", new Sequence() {
      public double getSValue() {

        return (averageBoldness);
      }
    });

    graph.addSequence("Vengefulness Average", new Sequence() {
      public double getSValue() {

        return (averageVengefulness);
      }
    });

    if (this.getMetaVengefulnessEqualToVengefulness()) {
    }
    else {
      graph.addSequence("MetaVengefulness Average", new Sequence() {
        public double getSValue() {

          return (averageMetaVengefulness);
        }
      });
    }

    graph.setXRange(0, 20);
    graph.setYRange(0, 10);
    graph.setAxisTitles("time", "Units");

//plot creation
    aPlot = new Plot("Sequence");
    aPlot.display();
    aPlot.setConnected(true);
    aPlot.setXRange(0, Math.pow(2, this.arrayDimension) - 1);
    aPlot.setYRange(0, Math.pow(2, this.arrayDimension) - 1);
    aPlot.setAxisTitles("Average Boldness ", "Average Vengefulness");

  }

  /**
   * This method establishes the sequence and the dynamics of the simulation
   */
  private void buildSchedule() {

    ActionGroup group = new ActionGroup(ActionGroup.SEQUENTIAL);
    group.createActionFor(this, "playGeneration");
    group.createActionFor(this, "computeTotalScores");
    group.createActionFor(this, "computePopulationAverages");
//In batch mode the model doesn't display any graph
    if (this.batchMode == false) {
      group.createActionFor(this, "display");
    }
//The model just write data if the parameter is activated
    if (this.getDataOutputFile()) {
      group.createActionFor(this, "writeData");
    }

    group.createActionFor(this, "geneticAlgorithm");

    schedule.scheduleActionBeginning(1, group);
//when we have to finish
    schedule.scheduleActionAt(this.numberOfGenerations, this, "stop");

  }

  /**
   * The begin method to determine what to build and what not
   */
  public void begin() {

    if (this.batchMode == true) {
      this.checkBatchparameters();

    }

    buildModel();
    if (this.batchMode == false) {
      buildDisplay();
    }
    buildSchedule();
    if (this.batchMode == false) {
      graph.display();
    }

  }

  /**
   * In this method we include the Axelrod's default parameters in the GUI mode
   */
  public void setup() {

    if (this.batchMode == false) {
      if (graph != null) {
        graph.dispose();
        if (aPlot != null) {
          aPlot.dispose();
        }

      }
      aPlot = null;
      graph = null;

      graph = new OpenSequenceGraph("Temporal evolution of agent properties", this);

      registerMediaProducer("Graph", graph);

    }

    schedule = new Schedule(1);

    normsAgentList = new ArrayList();

///Falta por generar el codigo de consistencia
    if (this.parameterFileName == null) {

      numAgents = 20;
      arrayDimension = 3;
      defection_T = 3;
      punishment_P = -9;
      hurtByOthers_H = -1;
      enforcementCost_E = -2;
      punishmentAgainstNonpunishment_PNP = -9;
      enforcementCostAgainstNonpunishment_ENP = -2;
      mutationRate_per_bit = 0.01;
      mutationRate_per_generation = 1.2;

      standardDeviationsForDoubleReproduction = 1;
      standardDeviationsForNoReproduction = -1;
      constantForRouletteWheel = 0;

      initialBoldnessAverage = (Math.pow(2, this.arrayDimension) - 1) /
          2;
      initialVengefulnessAverage = (Math.pow(2, this.arrayDimension) -
                                    1) / 2;
      initialMetaVengefulnessAverage = (Math.pow(2,
                                                 this.arrayDimension) - 1) / 2;

      rounds = 4;
      numberOfGenerations = 100000;
      averageScore = 0;
      deviation = 0;
      averageBoldness = 0;
      averageVengefulness = 0;
      averageMetaVengefulness = 0;
      initialization = "RANDOM";
      selection = "STANDARD_DEVIATION";
      activateMetanormsModel = true;
      metaVengefulnessEqualToVengefulness = true;
      paramOutputFile = true;
      dataOutputFile = true;
      outputStep = 10;

    }
    else {

    }

  }

  /**
   * Parameters
   * @return String[]
   */
  public String[] getInitParam() {
    String[] params = {
        "numAgents", "arrayDimension", "numberOfGenerations", "rounds",
        "mutationRate_per_bit", "mutationRate_per_generation", "defection_T",
        "punishment_P",
        "hurtByOthers_H", "enforcementCost_E",
        "punishmentAgainstNonpunishment_PNP",
        "enforcementCostAgainstNonpunishment_ENP", "initialBoldnessAverage",
        "initialVengefulnessAverage", "initialMetaVengefulnessAverage",
        "Pause", "Model",
        "paramOutputFile", "dataOutputFile", "activateMetanormsModel",
        "metaVengefulnessEqualToVengefulness", "initialization", "selection",
        "constantForRouletteWheel",
        "standardDeviationsForDoubleReproduction",
        "standardDeviationsForNoReproduction", "outputStep"};
    return params;

  }

  /**
   *
   * @return Schedule
   */
  public Schedule getSchedule() {
    return schedule;
  }

  /**
   *
   * @return String
   */
  public String getName() {
    return "NormsAgents";
  }

  /**
   *
   * @return String
   */
  public String getInitialization() {
    return this.initialization;
  }

  /**
   *
   * @param outputStep int
   */
  public void setOutputStep(int outputStep) {
    this.outputStep = outputStep;
  }

  /**
   *
   * @return int
   */
  public int getOutputStep() {
    return outputStep;
  }

  /**
   *
   * @return String
   */
  public String getSelection() {
    return this.selection;
  }

  /**
   *
   * @param selection String
   */
  public void setSelection(String selection) {
    this.selection = selection;
  }

  /**
   *
   * @param initialization String
   */
  public void setInitialization(String initialization) {

    this.initialization = initialization;

    if (this.batchMode == false) {

      //Checking the coherence of the initial values
      if (initialization.equals("RANDOM")) {
        this.setInitialBoldnessAverage( (Math.pow(2, this.arrayDimension) - 1) /
                                       2);
        this.setInitialVengefulnessAverage( (Math.pow(2, this.arrayDimension) -
                                             1) /
                                           2);
        this.setInitialMetaVengefulnessAverage( (Math.pow(2,
            this.arrayDimension) -
                                                 1) / 2);
      }

      if (this.getInitialBoldnessAverage() >
          (Math.pow(2, this.arrayDimension) - 1)) {
        this.setInitialBoldnessAverage( (Math.pow(2, this.arrayDimension) - 1));
      }
      else {
        if (this.getInitialBoldnessAverage() < 0) {
          this.setInitialBoldnessAverage(0);
        }
      }

      if (this.getInitialVengefulnessAverage() >
          (Math.pow(2, this.arrayDimension) - 1)) {
        this.setInitialVengefulnessAverage( (Math.pow(2, this.arrayDimension) -
                                             1));
      }
      else {
        if (this.getInitialVengefulnessAverage() < 0) {
          this.setInitialVengefulnessAverage(0);
        }
      }

      if (this.getInitialMetaVengefulnessAverage() >
          (Math.pow(2, this.arrayDimension) - 1)) {
        this.setInitialMetaVengefulnessAverage( (Math.pow(2,
            this.arrayDimension) -
                                                 1));
      }
      else {
        if (this.getInitialMetaVengefulnessAverage() < 0) {
          this.setInitialMetaVengefulnessAverage(0);
        }
      }

      if (this.getMetaVengefulnessEqualToVengefulness()) {
        this.setInitialMetaVengefulnessAverage(this.
                                               getInitialVengefulnessAverage());
      }

      this.initialization = initialization;

      ProbeUtilities.updateModelProbePanel();

    }

  }

  // properties
  public int getNumAgents() {
    return numAgents;
  }

  /**
   * This method set the number of agents in the model, and modifies consequently the  mutationRate_per_generation and mutationRate_per_generation parameters
   * @param numAgents int
   */
  public void setNumAgents(int numAgents) {
    this.numAgents = numAgents;

    if (this.batchMode == false) {

      if (this.metaVengefulnessEqualToVengefulness) {
        mutationRate_per_generation = this.mutationRate_per_bit * (numAgents *
            (arrayDimension * 2));
      }
      else {
        mutationRate_per_generation = this.mutationRate_per_bit * (numAgents *
            (arrayDimension * 3));
      }

      ProbeUtilities.updateModelProbePanel();

    }

  }

  /**
   *
   * @return int
   */
  public int getNumberOfGenerations() {
    return numberOfGenerations;
  }

  /**
   *
   * @param numberOfGenerations int
   */
  public void setNumberOfGenerations(int numberOfGenerations) {
    this.numberOfGenerations = numberOfGenerations;
  }

  /**
   *
   * @return int
   */
  public int getRounds() {
    return rounds;
  }

  /**
   *
   * @param rounds int
   */
  public void setRounds(int rounds) {
    this.rounds = rounds;
  }

  /**
   *
   * @return int
   */
  public int getArrayDimension() {
    return this.arrayDimension;
  }

  /**
   *
   * @param arrayDimension int
   */
  public void setArrayDimension(int arrayDimension) {

    this.arrayDimension = arrayDimension;

    if (this.batchMode == false) {

      if (initialization.equals("RANDOM")) {
        initialBoldnessAverage = ( (Math.pow(2, this.arrayDimension) - 1) /
                                  2);
        initialVengefulnessAverage = ( (Math.pow(2, this.arrayDimension) - 1) /
                                      2);
        initialMetaVengefulnessAverage = ( (Math.pow(2, this.arrayDimension) -
                                            1) / 2);
      }

      if (this.metaVengefulnessEqualToVengefulness) {
        mutationRate_per_generation = this.mutationRate_per_bit * (numAgents *
            (arrayDimension * 2));
      }
      else {
        mutationRate_per_generation = this.mutationRate_per_bit * (numAgents *
            (arrayDimension * 3));
      }

      ProbeUtilities.updateModelProbePanel();

    }

  }

  /**
   *
   * @return double
   */
  public double getDefection_T() {
    return this.defection_T;
  }

  /**
   *
   * @param defection_T double
   */
  public void setDefection_T(double defection_T) {
    this.defection_T = defection_T;
  }

  /**
   *
   * @return double
   */
  public double getMutationRate_per_bit() {
    return this.mutationRate_per_bit;
  }

  /**
   * This method modifies linearly the mutationRate_per_generation parameter (see information of the variable)
   * @param mutationRate_per_bit double
   */
  public void setMutationRate_per_bit(double mutationRate_per_bit) {
    this.mutationRate_per_bit = mutationRate_per_bit;

    if (this.batchMode == false) {

      if (this.metaVengefulnessEqualToVengefulness) {
        mutationRate_per_generation = this.mutationRate_per_bit * (numAgents *
            (arrayDimension * 2));
      }
      else {
        mutationRate_per_generation = this.mutationRate_per_bit * (numAgents *
            (arrayDimension * 3));

      }

      ProbeUtilities.updateModelProbePanel();

    }

  }

  /**
   *
   * @return double
   */
  public double getMutationRate_per_generation() {
    return this.mutationRate_per_generation;
  }

  /**
   * This method modifies linearly the mutationRate_per_bit parameter (see information of the variable)
   * @param mutationRate_per_generation double
   */
  public void setMutationRate_per_generation(double mutationRate_per_generation) {
    this.mutationRate_per_generation = mutationRate_per_generation;

    if (this.batchMode == true) {

      System.out.println(
          "Remember that in batch mode you should set the mutationRate_per_bit parameter");
      System.out.println("and not mutationRate_per_generation ");
      System.out.println("Try it again!!! ");
      System.exit(0);
    }

    if (this.batchMode == false) {

      if (this.metaVengefulnessEqualToVengefulness) {
        this.mutationRate_per_bit = mutationRate_per_generation / (numAgents *
            arrayDimension * 2);
      }
      else {
        this.mutationRate_per_bit = mutationRate_per_generation / (numAgents *
            arrayDimension * 3);

      }

      ProbeUtilities.updateModelProbePanel();

    }

  }

  /**
   *
   * @return double
   */
  public double getStandardDeviationsForDoubleReproduction() {
    return this.standardDeviationsForDoubleReproduction;
  }

  /**
   *
   * @param standardDeviationsForDoubleReproduction double
   */
  public void setStandardDeviationsForDoubleReproduction(double
      standardDeviationsForDoubleReproduction) {
    this.standardDeviationsForDoubleReproduction =
        standardDeviationsForDoubleReproduction;
    if (standardDeviationsForDoubleReproduction != 1.0) {
      System.out.println(
          "standardDeviationsForDoubleReproduction has to be equal to 1.0 in this version of RAEAN");
      System.exit(0);
    }
  }

  /**
   *
   * @return double
   */
  public double getConstantForRouletteWheel() {
    return this.constantForRouletteWheel;
  }

  /**
   *
   * @param constantForRouletteWheel double
   */
  public void setConstantForRouletteWheel(double
                                          constantForRouletteWheel) {
    this.constantForRouletteWheel =
        constantForRouletteWheel;
  }

  /**
   *
   * @return double
   */
  public double getStandardDeviationsForNoReproduction() {
    return this.standardDeviationsForNoReproduction;
  }

  /**
   *
   * @param standardDeviationsForNoReproduction double
   */
  public void setStandardDeviationsForNoReproduction(double
      standardDeviationsForNoReproduction) {
    this.standardDeviationsForNoReproduction =
        standardDeviationsForNoReproduction;
    if (standardDeviationsForNoReproduction != -1.0) {
      System.out.println(
          "standardDeviationsForNoReproduction has to be equal to -1.0 in this version of RAEAN");
      System.exit(0);
    }
  }

  /**
   *
   * @return double
   */
  public double getPunishment_P() {
    return this.punishment_P;
  }

  /**
   *
   * @param punishment_P double
   */
  public void setPunishment_P(double punishment_P) {
    this.punishment_P = punishment_P;
  }

  /**
   *
   * @return double
   */
  public double getInitialBoldnessAverage() {
    return this.initialBoldnessAverage;
  }

  /**
   *
   * @param initialBoldnessAverage double
   */
  public void setInitialBoldnessAverage(double initialBoldnessAverag) {

    this.initialBoldnessAverage = initialBoldnessAverag;

    if (this.batchMode == false) {

      //Checking the coherence of the initial values
      if (initialization.equals("RANDOM")) {
        initialBoldnessAverage = ( (Math.pow(2, this.arrayDimension) - 1) /
                                  2);
        initialVengefulnessAverage = ( (Math.pow(2, this.arrayDimension) - 1) /
                                      2);
        initialMetaVengefulnessAverage = ( (Math.pow(2, this.arrayDimension) -
                                            1) / 2);
      }

      if (initialBoldnessAverage >
          (Math.pow(2, this.arrayDimension) - 1)) {
        initialBoldnessAverage = ( (Math.pow(2, this.arrayDimension) - 1));
      }
      else {
        if (initialBoldnessAverage < 0) {
          initialBoldnessAverage = 0;
        }
      }

//Just integer values are available
      if (initialization.equals("CUSTOM")) {

        this.initialBoldnessAverage = (int) initialBoldnessAverag;
      }

      ProbeUtilities.updateModelProbePanel();

    }

  }

  /**
   *
   * @return double
   */
  public double getInitialVengefulnessAverage() {
    return this.initialVengefulnessAverage;
  }

  /**
   *
   * @param initialVengefulnessAverage double
   */
  public void setInitialVengefulnessAverage(double initialVengefulnessAverag) {

    this.initialVengefulnessAverage = initialVengefulnessAverag;

    if (this.batchMode == false) {

      //Comprobacin de la coherencia de los valores iniciales
      if (initialization.equals("RANDOM")) {
        initialBoldnessAverage = ( (Math.pow(2, this.arrayDimension) - 1) /
                                  2);
        initialMetaVengefulnessAverage = ( (Math.pow(2, this.arrayDimension) -
                                            1) /
                                          2);
        initialVengefulnessAverage = ( (Math.pow(2, this.arrayDimension) -
                                        1) / 2);
      }

      if (initialVengefulnessAverage >
          (Math.pow(2, this.arrayDimension) - 1)) {
        initialVengefulnessAverage = ( (Math.pow(2, this.arrayDimension) - 1));
      }
      else {
        if (initialVengefulnessAverage < 0) {
          initialVengefulnessAverage = 0;
        }
      }

      if (this.getMetaVengefulnessEqualToVengefulness()) {
        this.setInitialMetaVengefulnessAverage(initialVengefulnessAverage);
      }
//Just integer values are available
      if (initialization.equals("CUSTOM")) {
        this.initialVengefulnessAverage = (int) initialVengefulnessAverag;
      }

      ProbeUtilities.updateModelProbePanel();

    }

  }

  /**
   *
   * @return double
   */
  public double getInitialMetaVengefulnessAverage() {
    return this.initialMetaVengefulnessAverage;
  }

  /**
   *
   * @param initialMetaVengefulnessAverage double
   */
  public void setInitialMetaVengefulnessAverage(double
                                                initialMetaVengefulnessAverag) {

    this.initialMetaVengefulnessAverage = initialMetaVengefulnessAverag;

    if (this.batchMode == false) {

      //Comprobacin de la coherencia de los valores iniciales
      if (initialization.equals("RANDOM")) {
        initialBoldnessAverage = ( (Math.pow(2, this.arrayDimension) - 1) /
                                  2);
        initialVengefulnessAverage = ( (Math.pow(2, this.arrayDimension) - 1) /
                                      2);
        initialMetaVengefulnessAverage = ( (Math.pow(2, this.arrayDimension) -
                                            1) / 2);
      }

      if (initialMetaVengefulnessAverage >
          (Math.pow(2, this.arrayDimension) - 1)) {
        initialMetaVengefulnessAverage = ( (Math.pow(2, this.arrayDimension) -
                                            1));
      }
      else {
        if (initialMetaVengefulnessAverage < 0) {
          initialMetaVengefulnessAverage = 0;
        }
      }

      if (this.getMetaVengefulnessEqualToVengefulness()) {
        initialMetaVengefulnessAverage = (this.getInitialVengefulnessAverage());
      }
//Just integer values are available
      if (initialization.equals("CUSTOM")) {
        this.initialBoldnessAverage = (int)this.initialBoldnessAverage;
        this.initialVengefulnessAverage = (int)this.initialVengefulnessAverage;
        this.initialMetaVengefulnessAverage = (int)
            initialMetaVengefulnessAverag;
      }

      ProbeUtilities.updateModelProbePanel();

    }
  }

  /**
   *
   * @return double
   */
  public double getPunishmentAgainstNonpunishment_PNP() {
    return this.punishmentAgainstNonpunishment_PNP;
  }

  /**
   *
   * @param punishmentAgainstNonpunishment_PNP double
   */
  public void setPunishmentAgainstNonpunishment_PNP(double
      punishmentAgainstNonpunishment_PNP) {
    this.punishmentAgainstNonpunishment_PNP =
        punishmentAgainstNonpunishment_PNP;
  }

  /**
   *
   * @return double
   */
  public double getHurtByOthers_H() {
    return this.hurtByOthers_H;
  }

  /**
   *
   * @param hurtByOthers_H double
   */
  public void setHurtByOthers_H(double hurtByOthers_H) {
    this.hurtByOthers_H = hurtByOthers_H;
  }

  /**
   *
   * @return double
   */
  public double getEnforcementCost_E() {
    return this.enforcementCost_E;
  }

  /**
   *
   * @param enforcementCost_E double
   */
  public void setEnforcementCost_E(double enforcementCost_E) {
    this.enforcementCost_E = enforcementCost_E;
  }

  /**
   *
   * @return double
   */
  public double getEnforcementCostAgainstNonpunishment_ENP() {
    return this.enforcementCostAgainstNonpunishment_ENP;
  }

  /**
   *
   * @param enforcementCostAgainstNonpunishment_ENP double
   */
  public void setEnforcementCostAgainstNonpunishment_ENP(double
      enforcementCostAgainstNonpunishment_ENP) {
    this.enforcementCostAgainstNonpunishment_ENP =
        enforcementCostAgainstNonpunishment_ENP;
  }

  /**
   *
   * @return boolean
   */
  public boolean getParamOutputFile() {
    return paramOutputFile;
  }

  /**
   *
   * @param paramOutputFile boolean
   */
  public void setParamOutputFile(boolean paramOutputFile) {
    this.paramOutputFile = paramOutputFile;
  }

  /**
   *
   * @return boolean
   */
  public boolean getDataOutputFile() {
    return dataOutputFile;
  }

  /**
   *
   * @param dataOutputFile boolean
   */
  public void setDataOutputFile(boolean dataOutputFile) {
    this.dataOutputFile = dataOutputFile;
  }

  /**
   *
   * @return boolean
   */
  public boolean getActivateMetanormsModel() {
    return activateMetanormsModel;
  }

  /**
   *
   * @param activateMetanormsModel boolean
   */
  public void setActivateMetanormsModel(boolean activateMetanormsModel) {
    this.activateMetanormsModel = activateMetanormsModel;
  }

  /**
   *
   * @return boolean
   */
  public boolean getMetaVengefulnessEqualToVengefulness() {
    return metaVengefulnessEqualToVengefulness;
  }

  /**
   *
   * @param metaVengefulnessEqualToVengefulness boolean
   */
  public void setMetaVengefulnessEqualToVengefulness(boolean
      metaVengefulnessEqualToVengefulness) {
    this.metaVengefulnessEqualToVengefulness =
        metaVengefulnessEqualToVengefulness;

    if (this.batchMode == false) {

      if (this.metaVengefulnessEqualToVengefulness) {
        mutationRate_per_generation = this.mutationRate_per_bit * (numAgents *
            (arrayDimension * 2));
      }
      else {
        mutationRate_per_generation = this.mutationRate_per_bit * (numAgents *
            (arrayDimension * 3));
      }

      ProbeUtilities.updateModelProbePanel();

    }

  }

  /**
   * This method chooses the selected selection mechanism
   */
  public void geneticAlgorithm() {

    if (this.selection.equals("STANDARD_DEVIATION")) {
      this.computeDeviation();
    }
    else
    if (this.selection.equals("ROULETTE_WHEEL")) {
      this.rouletteWheel();
    }
    else {
      if (this.selection.equals("RANDOM_TOURNAMENT")) {
        this.randomTournament();
      }
      else {
        System.out.println("ERROR IN SELECTION METHOD");
      }
    }

  }

  /**
   * This method mutates an agent according to a mutationRate_per_bit
   * @param parent NormsAgent
   * @param mutationRate_per_bit double
   * @return NormsAgent
   */
  public NormsAgent mutation(NormsAgent parent, double mutationRate_per_bit) {
    NormsAgent offspring;

    offspring = new NormsAgent(parent, mutationRate_per_bit);

    return offspring;
  }

  /**
   * A method to write the params in a file
   */
  public void writeParams() {
    /**
     * To write in the parameters file
     */
    FileWriter fw;
    /**
     * To write in the parameters file
     */
    BufferedWriter bw;
    /**
     * Parameter file
     */
    PrintWriter salida;

    try {
      fw = new FileWriter(this.parametersUsed);
      bw = new BufferedWriter(fw);
      salida = new PrintWriter(bw);

      salida.println("Random Seed" + "," + this.getRngSeed());
      salida.println("Number of Agents" + "," + this.numAgents);
      salida.println("Number of Generations" + "," + this.numberOfGenerations);
      salida.println("Rounds" + "," + this.rounds);
      salida.println("Array Dimension" + "," + this.arrayDimension);
      salida.println("Mutation Rate (per bit)" + "," +
                     this.mutationRate_per_bit);
      salida.println("Mutation Rate (per generation)" + "," +
                     this.mutationRate_per_generation);
      salida.println("Defection T" + "," + this.defection_T);
      salida.println("Enforcement Cost E" + "," + this.enforcementCost_E);
      salida.println("Hurt by Others H" + "," + this.hurtByOthers_H);
      salida.println("Punishment P" + "," + this.punishment_P);
      salida.println("Enforcement Cost Against Non Punishment ENP" + "," +
                     this.enforcementCostAgainstNonpunishment_ENP);
      salida.println("Punishment Against Non Punishment PNP" + "," +
                     this.punishmentAgainstNonpunishment_PNP);
      salida.println("Initialization" + "," + this.initialization);
      salida.println("Selection" + "," + this.selection);
      salida.println("ActivateMetanormsModel" + "," +
                     this.activateMetanormsModel);
      salida.println("MetaVengefulnessEqualToVengefulness" + "," +
                     this.metaVengefulnessEqualToVengefulness);
      salida.println("OutputStep" + "," + this.outputStep);

      if (this.selection.equals("ROULETTE_WHEEL")) {
        salida.println("constantForRouletteWheel" + "," +
                       this.constantForRouletteWheel);
      }
      if (this.selection.equals("STANDARD_DEVIATION")) {
        salida.println("standardDeviationsForDoubleReproduction" + "," +
                       this.standardDeviationsForDoubleReproduction);
        salida.println("standardDeviationsForNoReproduction" + "," +
                       this.standardDeviationsForNoReproduction);
      }

      if (this.initialization.equals("CUSTOM")) {
        salida.println("initialBoldnessAverage" + "," +
                       this.initialBoldnessAverage);
        salida.println("initialVengefulnessAverage" + "," +
                       this.initialVengefulnessAverage);
        salida.println("initialMetaVengefulnessAverage" + "," +
                       this.initialMetaVengefulnessAverage);
      }

      salida.close();

    }

    catch (java.io.IOException e) {
      System.err.println("Exception writing Parameters");
    }

  }

  /**
   * Method to store the output data of the simulation
   */
  public void writeData() {

    if (super.getTickCount() - 1 == 0 ||
        ( (super.getTickCount() - 1) % (outputStep) == 0)) {
      salida2.print(this.averageBoldness + "\t");

      if (this.getMetaVengefulnessEqualToVengefulness()) {
        salida2.println(this.averageVengefulness);
      }
      else {
        salida2.print(this.averageVengefulness + "\t");
        salida2.println(this.averageMetaVengefulness);
      }
    }
    else {}

  }

  /**
   * To close the file
   */
  public void stop() {
    if (salida2 != null)
      salida2.close();

    super.stop();
  }

  /**
   * A method to create the names of the parameter and output files
   */
  public void createNameFile() {

    long runTime = java.lang.System.currentTimeMillis();

    int a=0;

    /*"Return from the systems time() function"*/
    Date today = new Date(runTime); /*"a verbose description of current time"*/
    String timeString = new String(today.toString());

    String paramFileName;
    String paramFileName2;

    timeString = timeString.replace(':', '_');
    timeString = timeString.replace(' ', '_');

    paramFileName = new String("Inputs_");
    paramFileName2 = new String("Outputs_");

    paramFileName = paramFileName.concat(timeString);
    paramFileName = paramFileName.concat("_0"+a);
    paramFileName = paramFileName.concat(".csv");

    paramFileName2 = paramFileName2.concat(timeString);
     paramFileName2 = paramFileName2.concat("_0"+a);
    paramFileName2 = paramFileName2.concat(".txt");

    this.parametersUsed = new String(paramFileName);
    this.outputFileName = new String(paramFileName2);

    if (this.getParamOutputFile()) {
      try {

        File f = new File(this.parametersUsed);

        while (f.exists()) {
          a++;
          paramFileName = paramFileName.replaceAll((a-1)+".csv",a+".csv");

          paramFileName2 = paramFileName2.replaceAll((a-1)+".txt",a+".txt");

          this.parametersUsed = new String(paramFileName);
          this.outputFileName = new String(paramFileName2);
f = new File(this.parametersUsed);

        }

        fw2 = new FileWriter(this.outputFileName);
        bw2 = new BufferedWriter(fw2);
        salida2 = new PrintWriter(bw2);

      }
      catch (java.io.IOException e) {
        System.err.println("Exception writing Parameters");
      }

    }

  }

  /**
   * Method to play a complete generation in the game
   */
  public void playGeneration() {

//For each round
    for (int s = 0; s < rounds; s++) {
      //Initialise every agents score
      for (int i = 0; i < normsAgentList.size(); i++) {
        NormsAgent agent = (NormsAgent) normsAgentList.get(i);
        agent.score[s] = 0;
      }

//Each agent has a chance to defect
      for (int i = 0; i < normsAgentList.size(); i++) {
        NormsAgent agent = (NormsAgent) normsAgentList.get(i);
        agent.defection();
      }

//Update defection-related scores

      for (int i = 0; i < normsAgentList.size(); i++) {

        NormsAgent agent = (NormsAgent) normsAgentList.get(i);
        if (agent.getDefection() == true) {
          agent.score[s] = agent.score[s] + this.defection_T;

          for (int j = 0; j < normsAgentList.size(); j++) {
            if (j != i) {
              NormsAgent otherAgent = (NormsAgent) normsAgentList.get(j);
              otherAgent.score[s] = otherAgent.score[s] + this.hurtByOthers_H;
            }
          }
        }
      }
      //Check whether theyve seen you or not and, if so, whether they punish
      //you or not

      for (int i = 0; i < normsAgentList.size(); i++) {
        NormsAgent agent = (NormsAgent) normsAgentList.get(i);
        //Only agents who defected from now on
        if (agent.getDefection() == true) {
          //Now we loop through the rest of the agents
          for (int j = 0; j < normsAgentList.size(); j++) {
            if (j != i) {

              NormsAgent otherAgent = (NormsAgent) normsAgentList.get(j);

              if (Uniform.staticNextDoubleFromTo(0,
                                                 Math.pow(2,
                  this.arrayDimension) -
                                                 1) < agent.getVariable_S()) {
                //Youve been framed!
                if (Uniform.staticNextIntFromTo(1,
                                                (int) (Math.pow(2,
                    this.arrayDimension) - 1)) <= //To avoid floating point errors
                    otherAgent.getVengefulnessDecimal())
                //And theyre going to punish you!
                {
                  //Youve been punished...
                  agent.score[s] = agent.score[s] + this.punishment_P;

                  //... but the supergrass j loses something too!
                  otherAgent.score[s] = otherAgent.score[s] +
                      this.enforcementCost_E;
                }
                else {
                  //and only if there are metanorms...
                  if (this.getActivateMetanormsModel() == true) {

                    //j saw i but he didnt punish him, and k might
                    //be looking...
                    for (int k = 0; k < normsAgentList.size(); k++) {
                      if (k != i & k != j) {

                        //For each k different from i and j...
                        NormsAgent agentMetaAvenging = (NormsAgent)
                            normsAgentList.get(k);

                        if (Uniform.staticNextDoubleFromTo(0,
                            Math.pow(2, this.arrayDimension) - 1) <
                            agent.getVariable_S()) {
                          //k has seen j not punishing

                          //Assuming Vengefulness is equal to MetaVengefulness...
                          if (this.getMetaVengefulnessEqualToVengefulness() == true) {

                            if (Uniform.staticNextIntFromTo(1,
                                (int) (Math.pow(2, this.arrayDimension) - 1)) <= //To avoid floating point errors
                                agentMetaAvenging.getVengefulnessDecimal())
                            //If k metapunishes j
                            {
                              //j receives the appropriate punishment...
                              otherAgent.score[s] = otherAgent.score[s] +
                                  this.punishmentAgainstNonpunishment_PNP;

                              //... but supergrass k suffers too!
                              agentMetaAvenging.score[s] = agentMetaAvenging.
                                  score[s] +
                                  this.enforcementCostAgainstNonpunishment_ENP;
                            }
                          } // End of Assuming Vengefulness is equal to ...
                          else {
                            //Assuming Vengefulness is NOT equal to MetaVengefulness...
                            if (this.getMetaVengefulnessEqualToVengefulness() == false) {

                              if (Uniform.staticNextIntFromTo(1,
                                  (int) (Math.pow(2, this.arrayDimension) - 1)) <= //To avoid floating point errors
                                  agentMetaAvenging.getMetaVengefulnessDecimal())
                              //If k metapunishes j
                              {
                                //j receives the appropriate punishment...
                                otherAgent.score[s] = otherAgent.score[s] +
                                    this.punishmentAgainstNonpunishment_PNP;

                                //... but supergrass k suffers too!
                                agentMetaAvenging.score[s] = agentMetaAvenging.
                                    score[s] +
                                    this.
                                    enforcementCostAgainstNonpunishment_ENP;
                              }
                            }
                            else {
                              System.out.println("ERROR");
                            }
                          } //END of Assuming Vengefulness is NOT equal ...
                        } //END of k has seen j not punishing
                      }
                    } //END of for each potential metapunisher
                  }
                }
              }
            }
          }
        }
      }

    }
//End of generation (s rounds). Scores computed

  }

  /**
   * After a generation is played, it is necessary to compute the average score of the population
   */
  public void computeTotalScores() {
    this.sumaTotal = 0.0;
    double puntosDelAgenteEnLasRondas = 0;

    for (int i = 0; i < this.normsAgentList.size(); i++) {
      puntosDelAgenteEnLasRondas = 0;
      NormsAgent tempAgent = (NormsAgent) normsAgentList.get(i);
      for (int j = 0; j < this.rounds; j++) {

        puntosDelAgenteEnLasRondas = tempAgent.score[j] +
            puntosDelAgenteEnLasRondas;
      }
      this.sumaTotal = this.sumaTotal + puntosDelAgenteEnLasRondas;
      tempAgent.totalScoreInTheRounds = puntosDelAgenteEnLasRondas;
    }

    //At this point we know each agents score and the total.
    //Average score of the population in this generation
    this.averageScore = this.sumaTotal / this.normsAgentList.size();

  }

  /**
   * In Axelrods original implementation the selection mechanism was as
   * follows: agents with a payoff exceeding the population average by at
   * least one standard deviation are replicated twice; agents who are at
   * least one standard deviation below the population average are eliminated;
   * and the rest of the agents are replicated once. The number of agents is
   * kept constant, but Axelrod does not specify exactly how so we have
   * implemented our mechanism.
   *
   * --LUIS--
   * I have reimplemented this method to avoid floating-point errors when
   * the payoffs are integers. It assumes that
   * standardDeviationsForDoubleReproduction = 1, and that
   * standardDeviationForNoReproduction = -1.
   */
  public void computeDeviation() {

    ArrayList offspringList = new ArrayList();
    offspringList.clear();

    double sumatorioLuis = 0.0;
    int nOfAgents = this.normsAgentList.size();

    //Square sum
    for (int i = 0; i < this.normsAgentList.size(); i++) {
      NormsAgent tempAgent = (NormsAgent) normsAgentList.get(i);
      sumatorioLuis += Math.pow(nOfAgents * tempAgent.totalScoreInTheRounds - this.sumaTotal, 2);
    }

    for (int i = 0; i < nOfAgents; i++) {
      NormsAgent tempAgent = (NormsAgent) normsAgentList.get(i);
      double tmpPayoff = 0.0;

      tmpPayoff = nOfAgents * Math.pow(nOfAgents * tempAgent.totalScoreInTheRounds - this.sumaTotal,2);
      //agents with a payoff exceeding the population average by at least "standardDeviationForDoubleReproductionTemp" standard deviation are replicated twice;
      if (tempAgent.totalScoreInTheRounds * nOfAgents >= this.sumaTotal && tmpPayoff >= sumatorioLuis) {
        offspringList.add(mutation(tempAgent, mutationRate_per_bit));
        offspringList.add(mutation(tempAgent, mutationRate_per_bit));
      }
      else{
          //agents who are at least "standardDeviationForNoReproductionTemp" standard deviation below the population average are eliminated; and the rest of the agents are replicated once
          if (tempAgent.totalScoreInTheRounds * nOfAgents >= this.sumaTotal ||
              (tempAgent.totalScoreInTheRounds * nOfAgents <= this.sumaTotal && tmpPayoff <= sumatorioLuis)) {
            offspringList.add(mutation(tempAgent, mutationRate_per_bit));
          }
      }
    }

    /*
     The method used in our implementation to keep the number of agents constant is as follow:
     After the selection mechanism has been executed, we may find that the number of offspring is not the same than the desired number of agents, so our solution is:
          1.	What is the number of offspring agents
          2.	Is that number the same than the desired number
     a.	Yes. Great!!! The algorithm has finished.
     b.	No, there are less offspring than parents. The algorithm chooses randomly one offspring and duplicates it. Check if we have the same number, if yes, finished, if no go again to 2.b
     c.	No, there are more offspring than parents. The algorithm chooses randomly one offspring and eliminates it. Check if we have the same number, if yes, finished, if no go again to 2.c
     */

    if (offspringList.size() < nOfAgents) {
      while (offspringList.size() < normsAgentList.size()) {
        int s = 0;
        s = Uniform.staticNextIntFromTo(0, offspringList.size() - 1);
        offspringList.add(mutation( ( (NormsAgent) offspringList.get(s)), 0)); //Puede dar lugar a una prdida de diversidad
      }
    }
    else {
      if (offspringList.size() > nOfAgents) {
        while (offspringList.size() > normsAgentList.size()) {
          int s = 0;
          s = Uniform.staticNextIntFromTo(0, offspringList.size() - 1);
          offspringList.remove(s);
        }

      }

    }

    this.normsAgentList.clear();
    this.normsAgentList.addAll(offspringList);

  }

  /**
   * To calculate the population averages of each generation to represent or/and store them
   */
  public void computePopulationAverages() {

    //Boldness
    this.averageBoldness = 0;
    for (int i = 0; i < this.normsAgentList.size(); i++) {
      NormsAgent tempAgent = (NormsAgent) normsAgentList.get(i);
      averageBoldness = averageBoldness + tempAgent.getBoldnessDecimal();
    }
    this.averageBoldness = this.averageBoldness / this.normsAgentList.size();

    //Vengefulness
    this.averageVengefulness = 0;
    for (int i = 0; i < this.normsAgentList.size(); i++) {
      NormsAgent tempAgent = (NormsAgent) normsAgentList.get(i);
      averageVengefulness = averageVengefulness +
          tempAgent.getVengefulnessDecimal();
    }
    this.averageVengefulness = this.averageVengefulness /
        this.normsAgentList.size();

    //MetaVengefulness
    this.averageMetaVengefulness = 0;
    for (int i = 0; i < this.normsAgentList.size(); i++) {
      NormsAgent tempAgent = (NormsAgent) normsAgentList.get(i);
      averageMetaVengefulness = averageMetaVengefulness +
          tempAgent.getMetaVengefulnessDecimal();
    }
    this.averageMetaVengefulness = this.averageMetaVengefulness /
        this.normsAgentList.size();

  }

  public void display() {

    graph.step();
//We draw the plot
    aPlot.updateGraph();
    aPlot.plotPoint(this.averageBoldness, this.averageVengefulness, 1);

  }

  /**
   * Roulette wheel selection mechanism
   */
  public void rouletteWheel() {

    ArrayList offspringList = new ArrayList();
    offspringList.clear();

    double minScore, aScore, ticket;
    double[] roulette = new double[this.getNumAgents() + 1];
    double constant = this.getConstantForRouletteWheel();
    //Search the min score of the agent population
    minScore = ( (NormsAgent)this.normsAgentList.get(0)).totalScoreInTheRounds;
    for (int i = 1; i < this.normsAgentList.size(); i++) {
      aScore = ( (NormsAgent)this.normsAgentList.get(i)).totalScoreInTheRounds;
      if (aScore < minScore) {
        minScore = aScore;
      }
    }

    minScore = -minScore;

    /*We build a roulette where the portion of each option is proportional to its score
     normalized to the minimum score of all the agents, so the score of the worst agent will be
     always the value of the ConstantForRouletteWheel parameter
     */
    //hacemos una ruleta tal que la porcin de cada opcin sea proporcional a
    //a su puntuacin normalizada al valor mnimo de todos los
    //agentes y un parmatro constante de ruido, de tal manera que la puntuacin
    //del peor agente siempre tenga el valor del parmetro

    //The first value
    roulette[0] = 0;
    for (int i = 0; i < this.normsAgentList.size(); i++) {
      //The accumulated values
      roulette[i +
          1] = ( (NormsAgent)this.normsAgentList.get(i)).totalScoreInTheRounds +
          minScore + constant + roulette[i];
    }

    //We have the complete roulette, now we have to make the draw

    if (0 != roulette[roulette.length - 1]) { //Case different to constant equal to 0 and every agent with the same score
      for (int i = 0; i < this.normsAgentList.size(); i++) {

        do {
          ticket = Uniform.staticNextDoubleFromTo(roulette[0],
                                                  roulette[roulette.length - 1]);
        }
        while (ticket == roulette[roulette.length - 1]); //To not include this limit value
        for (int j = 0; j < roulette.length - 1; j++) {
          if (ticket < roulette[j + 1] && ticket >= roulette[j]) {
            offspringList.add(mutation( (NormsAgent)this.normsAgentList.get(j),
                                       mutationRate_per_bit));
          }
        }
      }
      this.normsAgentList.clear();
      this.normsAgentList.addAll(offspringList);

      if (this.normsAgentList.size() != this.getNumAgents()) {

        System.out.println("ERROR IN WHEEL");
      }
    }
    else {
      this.randomTournament();
    }
  }

  /**
   * Random tournament selection mechanism
   */
  public void randomTournament() {

    int k;
    ArrayList offspringList = new ArrayList();
    offspringList.clear();
// Two DIFFERENT agents are selected randomly
    for (int i = 0; i < this.getNumAgents(); i++) {
      int j = Uniform.staticNextIntFromTo(0, this.getNumAgents() - 1);

      do {
        k = Uniform.staticNextIntFromTo(0, this.getNumAgents() - 1);
      }
      while (k == j);

      NormsAgent knightA = (NormsAgent) normsAgentList.get(j);
      NormsAgent knightB = (NormsAgent) normsAgentList.get(k);

//We choose the best of the couple of agents, in case of tie, it is random
      //because the initial selection was random
      if (knightA.totalScoreInTheRounds > knightB.totalScoreInTheRounds) {

        offspringList.add(mutation(knightA, mutationRate_per_bit));
      }
      else {
        offspringList.add(mutation(knightB, mutationRate_per_bit));
      }

    }
    this.normsAgentList.clear();
    this.normsAgentList.addAll(offspringList);

    if (this.normsAgentList.size() != this.getNumAgents()) {

      System.out.println("ERROR IN TOURNAMENT");

    }

  }

  /**
   * Main function
   * @param args String[]
   */
  public static void main(String[] args) {
    SimInit init = new SimInit();
    NormsModel model = new NormsModel();

    boolean moreThanAFileName = false;
    boolean batchMode = false;

    String nameOfTheFile = null;

    for (int i = 0; i < args.length; i++) {
      char c = args[i].charAt(0);

      switch (c) {
        case '-':
          char c2 = args[i].charAt(1);
          switch (c2) {
            case 'b':
              batchMode = true;
              break;
            case 'h':
              System.out.println("");
              System.out.println("HELP");
              System.out.println("In the usage synpsis below, square brackets[] are used to denote an optional elemet and angle brackets <> to describe some value the user should provide");
              System.out.println(
                  "java NameOftheFile.jar -jar [-b] [-help] [<NameOfParametersFile>]");
              System.out.println("");
              System.out.println("The command-line options are:");
              System.out.println(
                  "-b to run the batch model, by default GUI model (without -b)");
              System.out.println("-help or -h to see the help message");
              System.out.println("<NameOfParametersFile> the name of the parameter file, if the batch mode is active, this parameter is required ");
              System.out.println("");
              break;
            default:

              if (moreThanAFileName == false) {
                nameOfTheFile = args[i];
                moreThanAFileName = true;

              }
              else {
                System.out.println("");
                System.out.println("You can just include a parameter file");
                System.out.println("HELP");
                System.out.println("In the usage synpsis below, square brackets[] are used to denote an optional elemet and angle brackets <> to describe some value the user should provide");
                System.out.println(
                    "java NameOftheFile.jar -jar [-b] [-help] [<NameOfParametersFile>]");
                System.out.println("");
                System.out.println("The command-line options are:");
                System.out.println(
                    "-b to run the batch model, by default GUI model (without -b)");
                System.out.println("-help or -h to see the help message");
                System.out.println("<NameOfParametersFile> the name of the parameter file, if the batch mode is active, this parameter is required ");
                System.out.println("");
              }

              break;
          }

          break;
        default:
          if (moreThanAFileName == false) {
            nameOfTheFile = args[i];
            moreThanAFileName = true;

          }
          else {
            System.out.println("");
            System.out.println("You can just include a parameter file");
            System.out.println("HELP");
            System.out.println("In the usage synpsis below, square brackets[] are used to denote an optional elemet and angle brackets <> to describe some value the user should provide");
            System.out.println(
                "java NameOftheFile.jar -jar [-b] [-help] [<NameOfParametersFile>]");
            System.out.println("");
            System.out.println("The command-line options are:");
            System.out.println(
                "-b to run the batch model, by default GUI model (without -b)");
            System.out.println("-help or -h to see the help message");
            System.out.println("<NameOfParametersFile> the name of the parameter file, if the batch mode is active, this parameter is required ");
            System.out.println("");
          }
          break;
      }

    }

    if (batchMode == true && nameOfTheFile == null) {
      System.out.println("");
      System.out.println(
          "if the batch mode is active, the parameter file is required");
      System.out.println("HELP");
      System.out.println("In the usage synpsis below, square brackets[] are used to denote an optional element and angle brackets <> to describe some value the user should provide");
      System.out.println(
          "java NameOftheFile.jar -jar [-b] [-help] [<NameOfParametersFile>]");
      System.out.println("");
      System.out.println("The command-line options are:");
      System.out.println(
          "-b to run the batch model, by default GUI model (without -b)");
      System.out.println("-help or -h to see the help message");
      System.out.println("<NameOfParametersFile> the name of the parameter file, if the batch mode is active, this parameter is required ");
      System.out.println("");

      System.exit(0);
    }
//Desactivar la comprobacin de los sets
    if (batchMode == false && nameOfTheFile != null) {
      System.out.println("");
      System.out.println("");
      System.out.println("Sorry, but this version does not allow to change the default parameters of the GUI mode from a file");
      System.out.println("If you want to change the default parameters and run the simulation in GUI mode you have to change the parameters in the GUI window");
      System.out.println("");
      System.exit(0);

    }

    if (args.length == 0) {
      model.batchMode = batchMode;

      init.loadModel(model, null, batchMode);

    }
    else {
      model.batchMode = batchMode;

      model.parameterFileName = nameOfTheFile;
      init.loadModel(model, nameOfTheFile, batchMode);

    }

  }

  public void checkBatchparameters() {

//mutationRate_per_generation is calculated
    if (this.metaVengefulnessEqualToVengefulness) {
      mutationRate_per_generation = this.mutationRate_per_bit * (numAgents *
          (arrayDimension * 2));
    }
    else {
      mutationRate_per_generation = this.mutationRate_per_bit * (numAgents *
          (arrayDimension * 3));
    }
//System.out.println("The mutationRate_per_generation of this simulation is "+mutationRate_per_generation);
    //System.out.println("Remember that in batch mode you should set the mutationRate_per_bit parameter");

//if initialization is RANDOM
    if (initialization.equals("RANDOM")) {
      if (initialBoldnessAverage != ( (Math.pow(2, this.arrayDimension) - 1) /
                                     2)) {
        initialBoldnessAverage = ( (Math.pow(2, this.arrayDimension) - 1) /
                                  2);
        System.out.println(
            "Initial boldness is RANDOM and with initial average value of " +
            initialBoldnessAverage);

      }

      if (initialVengefulnessAverage != ( (Math.pow(2, this.arrayDimension) - 1) /
                                         2)) {
        initialVengefulnessAverage = ( (Math.pow(2, this.arrayDimension) - 1) /
                                      2);
        System.out.println(
            "Initial vengefulness is RANDOM and with initial average value of " +
            initialVengefulnessAverage);

      }
      if (initialMetaVengefulnessAverage !=
          ( (Math.pow(2, this.arrayDimension) - 1) /
           2)) {
        initialMetaVengefulnessAverage = ( (Math.pow(2, this.arrayDimension) -
                                            1) /
                                          2);
        System.out.println(
            "Initial metavengefulness is RANDOM and with initial average value of " +
            initialMetaVengefulnessAverage);

      }

    }
//if initialization is CUSTOM

    if (initialization.equals("CUSTOM")) {
      if (initialBoldnessAverage != (int) initialBoldnessAverage) {
        initialBoldnessAverage = (int) initialBoldnessAverage;
        System.out.println(
            "Initial boldness of the population is an integer value = " +
            initialBoldnessAverage);

      }

      if (initialVengefulnessAverage != (int) initialVengefulnessAverage) {
        initialVengefulnessAverage = (int) initialVengefulnessAverage;
        System.out.println(
            "Initial vengefulness of the population is an integer value = " +
            initialVengefulnessAverage);

      }

      if (initialMetaVengefulnessAverage !=
          (int) initialMetaVengefulnessAverage) {
        initialMetaVengefulnessAverage = (int) initialMetaVengefulnessAverage;
        System.out.println(
            "Initial meta vengefulness of the population is an integer value = " +
            initialMetaVengefulnessAverage);

      }

      if (this.getMetaVengefulnessEqualToVengefulness()) {
        if (initialVengefulnessAverage != initialMetaVengefulnessAverage) {
          initialMetaVengefulnessAverage = initialVengefulnessAverage;
          System.out.println(
              "The parameter MetaVengefulnessEqualToVengefulness has been setup as true in this simulation, so the initialMetaVengefulnessAverage has been changed to " +
              initialVengefulnessAverage);

        }
      }

    }

  }

}
